@@ -200,7 +200,7 @@ edge_pos_pre_convert_hook (BasePWidget *basep)
 }
 
 GtkWidget *
-edge_widget_new (gchar *panel_id,
+edge_widget_new (const char *panel_id,
 		 int screen,
 		 int monitor,
 		 BorderEdge edge,
@@ -209,8 +209,8 @@ edge_widget_new (gchar *panel_id,
 		 int sz,
 		 gboolean hidebuttons_enabled,
 		 gboolean hidebutton_pixmaps_enabled,
-		 PanelBackType back_type,
-		 char *back_pixmap,
+		 PanelBackgroundType back_type,
+		 const char *back_pixmap,
 		 gboolean fit_pixmap_bg,
 		 gboolean stretch_pixmap_bg,
 		 gboolean rotate_pixmap_bg,
